import requests
from bs4 import BeautifulSoup
from pykitables.wikitable import Wikitable

def get_tables(url):

    html = requests.get(url).text
    soup = BeautifulSoup(html, 'html.parser')
    tables_list=soup.find_all('table', {'class': 'wikitable'})
    wikitables_list = list()

    for table in tables_list:
        rows = table.find_all('tr')
        wikitable = Wikitable()
        for row in rows:
            cells = row.find_all(['td', 'th'])
            if cells is None: #TODO?
                continue
            for cell in cells:
                rowspan = cell.get('rowspan')
                colspan = cell.get('colspan')
                value = cell.text.strip()
                if ',' in value or '\n' in value:
                    value = '"'+value+'"'
                if colspan is None:
                    colspan = 1
                if rowspan is None:
                    rowspan = 1
                for m in range(int(colspan)):
                    wikitable.add_value(value)
                for n in range(int(rowspan)-1):
                    wikitable.next_add(value, n+1)
            wikitable.iterate_row()
        wikitables_list.append(wikitable)

    return wikitables_list



from pykitables.wikirow import Wikirow

class Wikitable:

    def __init__(self):
        self.table = [Wikirow()]
        self.current_index = 0

    def add_value(self, value):
        try:
            self.table[self.current_index].add_cell(value)
        except IndexError:
            self.new_row()
            self.add_value(value)

    def iterate_row(self):
        self.current_index = self.current_index + 1

    def next_add(self, value, index):
        for i in range(len(self.table)-1, self.current_index+index):
            self.new_row()
        self.table[self.current_index+index].insert_cell(value, self.current_len())

    def new_row(self):
        self.table.append(Wikirow())

    def to_csv(self):
        csv = ""
        for row in self.table:
            csv = csv + row.to_csv()
            csv = csv+'\n'
        return csv

    #TODO: messy, replace with get current row perhaps
    def current_len(self):
        return self.table[self.current_index].current_len()



class Wikirow:

    def __init__(self):
        self.row = list()
        self.current_index = 0

    def add_cell(self, value):
        try:
            while self.row[self.current_index] is not None:
                self.current_index = self.current_index+1
            self.row[self.current_index] = value
            self.current_index = self.current_index+1
        except IndexError:
            self.row.append(None)
            self.add_cell(value)

    def insert_cell(self, value, index):
        for i in range(len(self.row), index-1):
            self.row.append(None)
        self.row.append(value)

    def to_csv(self):
        values = ""
        for val in self.row:
            values = values + str(val) + ","
        return values

    def current_len(self):
        return len(self.row)
